﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Text;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Http;
using static CompServ.CompServConst;

namespace CompServ
{
    public static class ModelExtensionMethod
    {
        public static async Task<CompressRequestModel> ExtractCompressRequestAsync(this HttpRequest request)
        {
            if (int.TryParse(request.Headers[RequestHeaderCompressThreads], out var cThread) == false) cThread = 1;
            if (int.TryParse(request.Headers[RequestHeaderCompressLevel], out var cLevel) == false) cLevel = 0;

            var msInput = request.ContentLength.HasValue
                ? new MemoryStream((int)request.ContentLength.Value)
                : new MemoryStream();

            request.Body.Position = 0;
            await request.Body.CopyToAsync(msInput);

            return new CompressRequestModel
            {
                Level = cLevel,
                Threads = cThread,
                DataToCompress = msInput.ToArray()
            };
        }

        public static HttpRequestMessage BuildRequestMessage(this CompressRequestModel model)
        {
            return new HttpRequestMessage(HttpMethod.Post, ApiPathWorkerCompress)
            {
                Headers =
                {
                    {RequestHeaderCompressLevel,model.Level.ToString()},
                    {RequestHeaderCompressThreads,model.Threads.ToString()},

                },
                Version = HttpVersion.Version20,
                VersionPolicy = HttpVersionPolicy.RequestVersionExact,
                Content = new ByteArrayContent(model.DataToCompress.Array!, model.DataToCompress.Offset, model.DataToCompress.Count)
            };
        }

        public static HttpRequestMessage BuildRequestMessage(this DecompressRequestModel model)
        {
            return new HttpRequestMessage(HttpMethod.Post, ApiPathWorkerDecompress)
            {
                Content = new ByteArrayContent(model.DataToDecompress.Array!, model.DataToDecompress.Offset, model.DataToDecompress.Count),
                Version = HttpVersion.Version20,
                VersionPolicy = HttpVersionPolicy.RequestVersionExact,
            };
        }
    }
}
